@@ -25,7 +25,9 @@ void unblock_completion();
 bool completion_visible();
 gboolean hide_completion();
 void toggle_completion_visible();
+bool activate_previous_completion();
 bool activate_first_completion();
+void reset_tabbed_completion();
 
 bool completion_enter_pressed();
 void completion_up_pressed();
@@ -33,6 +35,8 @@ void completion_down_pressed();
 
 void update_completion();
 
+std::string current_completion_object();
+
 void create_expression_completion();
 
 void completion_font_modified();
@@ -45,7 +49,7 @@ void set_expression_completion_settings(int enable1 = -1, int enable2 = -1, int
 bool read_expression_completion_settings_line(std::string &svar, std::string &svalue, int &v);
 void write_expression_completion_settings(FILE *file);
 
-void do_completion(bool to_menu = false);
+void do_completion(bool to_menu = false, bool force = false);
 void on_completion_match_selected(GtkTreeView*, GtkTreePath *path, GtkTreeViewColumn*, gpointer);
 
 #endif /* EXPRESSION_COMPLETION_H */
