@@ -110,6 +110,15 @@ size_t unformatted_length(const string &str) {
 	return l;
 }
 
+string replace_result_separators(string str) {
+	if(printops.digit_grouping == DIGIT_GROUPING_LOCALE && !evalops.parse_options.comma_as_separator && CALCULATOR->local_digit_group_separator == COMMA && printops.comma() == ";" && printops.decimalpoint() == ".") {
+		gsub(COMMA, "", str);
+	} else if(printops.digit_grouping == DIGIT_GROUPING_LOCALE && !evalops.parse_options.dot_as_separator && CALCULATOR->local_digit_group_separator == DOT && printops.decimalpoint() != DOT) {
+		gsub(DOT, "", str);
+	}
+	return str;
+}
+
 void remove_separator(string &copy_text) {
 	for(size_t i = ((CALCULATOR->local_digit_group_separator.empty() || CALCULATOR->local_digit_group_separator == " " || CALCULATOR->local_digit_group_separator == printops.decimalpoint()) ? 1 : 0); i < 4; i++) {
 		string str_sep;
@@ -182,9 +191,9 @@ bool equalsIgnoreCase(const string &str1, const string &str2, size_t i2, size_t
 	size_t l = 0;
 	if(i2_end == string::npos) i2_end = str2.length();
 	for(size_t i1 = 0;; i1++, i2++) {
-		if(i2 >= i2_end) {
-			return i1 >= str1.length();
-		}
+		if(i2 >= i2_end) return i1 >= str1.length();
+		if(i1 >= str1.length()) break;
+		if(i2 >= str2.length()) return false;
 		if(i1 >= str1.length()) break;
 		if(((signed char) str1[i1] < 0 && i1 + 1 < str1.length()) || ((signed char) str2[i2] < 0 && i2 + 1 < str2.length())) {
 			size_t iu1 = 1, iu2 = 1;
@@ -310,10 +319,6 @@ const gchar *key_press_get_symbol(GdkEventKey *event, bool do_caret_as_xor, bool
 		case GDK_KEY_plus: {
 			return "+";
 		}
-		case GDK_KEY_braceleft: {}
-		case GDK_KEY_braceright: {
-			return "";
-		}
 	}
 	return NULL;
 }
@@ -536,6 +541,30 @@ void set_tooltips_enabled(GtkWidget *w, bool b) {
 	}
 }
 
+bool hide_tooltips(GtkWidget *w) {
+	if(!w) return false;
+	if(gtk_widget_get_has_tooltip(w)) {
+		gtk_widget_set_has_tooltip(w, FALSE);
+		gtk_widget_trigger_tooltip_query(w);
+		gtk_widget_set_has_tooltip(w, TRUE);
+	}
+	if(GTK_IS_BIN(w)) {
+		return hide_tooltips(gtk_bin_get_child(GTK_BIN(w)));
+	} else if(GTK_IS_CONTAINER(w)) {
+		GList *list = gtk_container_get_children(GTK_CONTAINER(w));
+		for(GList *l = list; l != NULL; l = l->next) {
+			if(GTK_IS_WIDGET(l->data)) {
+				if(hide_tooltips(GTK_WIDGET(l->data))) {
+					g_list_free(list);
+					return true;
+				}
+			}
+		}
+		g_list_free(list);
+	}
+	return false;
+}
+
 void update_window_properties(GtkWidget *d, bool ignore_tooltips_setting) {
 	if(!ignore_tooltips_setting && (!enable_tooltips || toe_changed)) set_tooltips_enabled(d, enable_tooltips);
 	if(always_on_top || aot_changed) gtk_window_set_keep_above(GTK_WINDOW(d), always_on_top);
@@ -556,7 +585,7 @@ void show_message(const gchar *text, GtkWindow *win) {
 }
 bool ask_question(const gchar *text, GtkWindow *win) {
 	if(!win) win = main_window();
-	GtkWidget *edialog = gtk_message_dialog_new(win, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_YES_NO, "%s", text);
+	GtkWidget *edialog = gtk_message_dialog_new(win, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, "%s", text);
 	if(always_on_top) gtk_window_set_keep_above(GTK_WINDOW(edialog), always_on_top);
 	int question_answer = gtk_dialog_run(GTK_DIALOG(edialog));
 	gtk_widget_destroy(edialog);
@@ -1055,6 +1084,7 @@ string get_value_string(const MathStructure &mstruct_, int type, Prefix *prefix)
 		po.preserve_precision = true;
 		po.interval_display = INTERVAL_DISPLAY_PLUSMINUS;
 		if(type == 1) po.show_ending_zeroes = false;
+		else po.restrict_to_parent_precision = false;
 		if(po.number_fraction_format == FRACTION_DECIMAL) po.number_fraction_format = FRACTION_DECIMAL_EXACT;
 	}
 	string str = CALCULATOR->print(mstruct_, 100, po);
@@ -1196,12 +1226,11 @@ Unit *find_exact_matching_unit(const MathStructure &m) {
 void remove_non_units(MathStructure &m) {
 	if(m.isPower() && m[0].isUnit()) return;
 	if(m.size() > 0) {
-		for(size_t i = 0; i < m.size();) {
-			if(m[i].isFunction() || m[i].containsType(STRUCT_UNIT, true) <= 0) {
-				m.delChild(i + 1);
+		for(size_t i = m.size(); i > 0; i--) {
+			if(m[i - 1].isFunction() || m[i - 1].containsType(STRUCT_UNIT, true) <= 0) {
+				m.delChild(i);
 			} else {
-				remove_non_units(m[i]);
-				i++;
+				remove_non_units(m[i - 1]);
 			}
 		}
 		if(m.size() == 0) m.clear();
@@ -1215,21 +1244,27 @@ void find_matching_units(const MathStructure &m, const MathStructure *mparse, ve
 			if(v[i] == u) return;
 		}
 		v.push_back(u);
+		if(top && mparse) {
+			Unit *u2 = CALCULATOR->findMatchingUnit(*mparse);
+			if(u2 && u2 != u && u2->baseUnit() == u->baseUnit() && u2->baseExponent() == u->baseExponent() && u2->category() != u->category()) v.push_back(u2);
+		}
 		return;
 	}
-	if(top) {
-		if(mparse && !m.containsType(STRUCT_UNIT, true) && (mparse->containsFunctionId(FUNCTION_ID_ASIN) || mparse->containsFunctionId(FUNCTION_ID_ACOS) || mparse->containsFunctionId(FUNCTION_ID_ATAN))) {
-			v.push_back(CALCULATOR->getRadUnit());
-			return;
-		}
+	if(top && mparse && !m.containsType(STRUCT_UNIT, true) && (mparse->containsFunctionId(FUNCTION_ID_ASIN) || mparse->containsFunctionId(FUNCTION_ID_ACOS) || mparse->containsFunctionId(FUNCTION_ID_ATAN))) {
+		v.push_back(CALCULATOR->getRadUnit());
+		return;
+	}
+	if(top && m.containsType(STRUCT_UNIT, true) > 0 && m.size() < 100) {
 		MathStructure m2(m);
 		remove_non_units(m2);
 		CALCULATOR->beginTemporaryStopMessages();
-		m2 = CALCULATOR->convertToOptimalUnit(m2, evalops, evalops.auto_post_conversion == POST_CONVERSION_OPTIMAL_SI);
+		CALCULATOR->startControl(50);
+		m2 = CALCULATOR->convertToOptimalUnit(m2);
+		CALCULATOR->stopControl();
 		CALCULATOR->endTemporaryStopMessages();
 		find_matching_units(m2, mparse, v, false);
 	} else {
-		for(size_t i = 0; i < m.size(); i++) {
+		for(size_t i = 0; i < m.size() && i < 100; i++) {
 			if(!m.isFunction() || !m.function()->getArgumentDefinition(i + 1) || m.function()->getArgumentDefinition(i + 1)->type() != ARGUMENT_TYPE_ANGLE) {
 				find_matching_units(m[i], mparse, v, false);
 			}
@@ -1378,3 +1413,26 @@ unsigned int combo_get_bits(GtkComboBox *w, bool has_auto) {
 	}
 	return bits;
 }
+
+extern MathFunction *f_title;
+bool test_autocalculatable(const MathStructure &m, bool where, bool top) {
+	if(m.isFunction()) {
+		if(m.size() < (size_t) m.function()->minargs() && (!where || m.size() != 0) && (m.size() != 1 || m[0].representsScalar())) {
+			MathStructure mfunc(m);
+			mfunc.calculateFunctions(evalops, false);
+			return false;
+		}
+		if(m.function()->id() == FUNCTION_ID_SAVE || m.function()->id() == FUNCTION_ID_PLOT || m.function()->id() == FUNCTION_ID_EXPORT || m.function()->id() == FUNCTION_ID_LOAD || m.function()->id() == FUNCTION_ID_COMMAND || m.function() == f_title || (m.function()->subtype() == SUBTYPE_USER_FUNCTION && ((UserFunction*) m.function())->formula().find("plot(") != string::npos)) return false;
+		if(m.size() > 0 && (m.function()->id() == FUNCTION_ID_FACTORIAL || m.function()->id() == FUNCTION_ID_DOUBLE_FACTORIAL || m.function()->id() == FUNCTION_ID_MULTI_FACTORIAL) && m[0].isInteger() && m[0].number().integerLength() > 17) {
+			return false;
+		}
+		if(m.function()->id() == FUNCTION_ID_LOGN && m.size() == 2 && m[0].isUndefined() && m[1].isNumber()) return false;
+		if(top && m.function()->subtype() == SUBTYPE_DATA_SET && m.size() >= 2 && m[1].isSymbolic() && equalsIgnoreCase(m[1].symbol(), "info")) return false;
+	} else if(m.isSymbolic() && contains_plot_or_save(m.symbol())) {
+		return false;
+	}
+	for(size_t i = 0; i < m.size(); i++) {
+		if(!test_autocalculatable(m[i], where || (m.isFunction() && m.function()->id() == FUNCTION_ID_REPLACE && i > 0), false)) return false;
+	}
+	return true;
+}
